﻿using Syncfusion.Drawing;
using Syncfusion.Pdf;
using Syncfusion.Pdf.Graphics;
using Syncfusion.Pdf.Interactive;
using Syncfusion.Pdf.Parsing;
using Syncfusion.Pdf.Security;
using System;
using System.Collections.Generic;
using System.IO;

namespace InvalidSignature
{
    class Program
    {
        static void Main(string[] args)
        {
            CreatePdf("test.pdf");
            using (var fs = File.OpenRead("test.pdf"))
            {
                var doc = new PdfLoadedDocument(fs);
                SignPdf(doc, "field1", "password-123456.pfx", "123456");
                SignPdf(doc, "field2", "password-654321.pfx", "654321");

                using (var signedFs = File.Open("signed.pdf", FileMode.Create, FileAccess.ReadWrite))
                {
                    doc.Save(signedFs);
                    doc.Close(true);
                }
            }

            ValidatePdf("signed.pdf");
            Console.ReadKey();
        }

        private static void CreatePdf(string documentName)
        {
            var doc = new PdfDocument();
            doc.PageSettings.Size = PdfPageSize.A4;
            var page = doc.Pages.Add();
            var graphics = page.Graphics;
            var font = new PdfStandardFont(PdfFontFamily.Helvetica, 20);
            graphics.DrawString("Signatures:", font, PdfBrushes.Black, new PointF(0, 0));

            var field1 = new PdfSignatureField(page, "field1")
            {
                Bounds = new RectangleF(100, 100, 200, 100)
            };
            doc.Form.Fields.Add(field1);
            var field2 = new PdfSignatureField(page, "field2")
            {
                Bounds = new RectangleF(100, 300, 200, 100)
            };
            doc.Form.Fields.Add(field2);

            using (var fs = File.Open(documentName, FileMode.Create, FileAccess.ReadWrite))
            {
                doc.Save(fs);
                doc.Close(true);
            }
        }

        private static void SignPdf(PdfLoadedDocument document, string fieldName, string certname, string certpassword)
        {
            if (document.Form.Fields.TryGetField(fieldName, out var field))
            {
                if (field is PdfLoadedSignatureField signatureField)
                {
                    if (!signatureField.IsSigned)
                    {
                        using (var cs = File.OpenRead(certname))
                        {
                            var cert = new PdfCertificate(cs, certpassword);
                            var page = document.Pages[0] as PdfLoadedPage;
                            signatureField.Signature = new PdfSignature(document, page, cert, fieldName, signatureField)
                            {
                                DocumentPermissions = PdfCertificationFlags.AllowComments
                            };
                        }
                    }
                }
            }
        }

        private static void ValidatePdf(string document)
        {
            using (var fs = File.OpenRead(document))
            {
                var doc = new PdfLoadedDocument(fs);
                foreach (var field in doc.Form.Fields)
                {
                    if (field is PdfLoadedSignatureField signatureField)
                    {
                        if (signatureField.IsSigned)
                        {
                            var result = signatureField.ValidateSignature();
                            Console.WriteLine($"cert: = {signatureField.Name}");
                            Console.WriteLine($"  - IsDocumentModified = {result.IsDocumentModified}");
                            Console.WriteLine($"  - IsSignatureValid = {result.IsSignatureValid}");
                            Console.WriteLine($"  - IsCertificated = {result.IsCertificated}");
                            foreach (var error in result.SignatureValidationErrors ?? new List<Syncfusion.Pdf.Security.PdfSignatureValidationException>())
                            {
                                Console.WriteLine($"  ## error = {error.Message}");
                            }

                            Console.WriteLine();
                        }
                    }
                }

                doc.Close(true);
            }
        }
    }
}
